pub struct ReflectMessage(/* private fields */);Expand description
A struct used to operate on reflected Message trait of a type.
A ReflectMessage for type T can be obtained via
bevy_reflect::TypeRegistration::data.
Implementations
Sourceimpl ReflectMessage
impl ReflectMessage
Sourcepub fn write_message(
&self,
world: &mut World,
message: &(dyn PartialReflect + 'static),
registry: &TypeRegistry,
)
pub fn write_message( &self, world: &mut World, message: &(dyn PartialReflect + 'static), registry: &TypeRegistry, )
Triggers a reflected Message like write_message().
Sourcepub fn new(fns: ReflectMessageFns) -> ReflectMessage
pub fn new(fns: ReflectMessageFns) -> ReflectMessage
Create a custom implementation of ReflectMessage.
This is an advanced feature, useful for scripting implementations, that should not be used by most users unless you know what you are doing.
Usually you should derive Reflect and add the #[reflect(Message)]
attribute to generate a ReflectMessage implementation automatically.
See ReflectMessageFns for more information.
Sourcepub fn fn_pointers(&self) -> &ReflectMessageFns
pub fn fn_pointers(&self) -> &ReflectMessageFns
The underlying function pointers implementing methods on ReflectMessage.
This is useful when you want to keep track locally of an individual function pointer.
Calling TypeRegistry::get followed by
TypeRegistration::data::<ReflectMessage> can be costly if done several
times per frame. Consider cloning ReflectMessage and keeping it
between frames, cloning a ReflectMessage is very cheap.
If you only need a subset of the methods on ReflectMessage,
use fn_pointers to get the underlying ReflectMessageFns
and copy the subset of function pointers you care about.